'==============================================================================
' SOUND.BAS
' Tonausgabe und Musik
' Sound-Funktion von yetifoot aus dem FB-Forum 
' (http://www.freebasic.net/forum/viewtopic.php?p=20477)
' Siehe FB-Manual, Community Code Library - Sound - Using the PC Speaker by several
' Erstellt am 08.01.2011
'==============================================================================


DECLARE SUB Sound(ByVal freq As Uinteger, dur As Uinteger)
DECLARE SUB Play (Melodie AS STRING)


' Sound Function v0.3 For DOS/Linux/Win by yetifoot
'
' Tested on:
'    Slackware 10.2
'    Win98
'    WinXP
'    DOS
'
' Credits:
'    http://www.frontiernet.net/~fys/snd.htm
'    http://delphi.about.com/cs/adptips2003/a/bltip0303_3.htm
'
' Notes:
'    On windows >= NT, direct port access is not allowed, in this instance
'    however we can use the WinAPI Beep function, which allows freq and duration
'    

'        Octave --  --    0    1    2    3    4    5 
'        Note
'        C     16   33   65  131  262  523 1046 2093
'        C#    17   35   69  139  277  554 1109 2217
'        D     18   37   73  147  294  587 1175 2349
'        D#    19   39   78  155  311  622 1244 2489
'        E     21   41   82  165  330  659 1328 2637
'        F     22   44   87  175  349  698 1397 2794
'        F#    23   46   92  185  370  740 1480 2960
'        G     24   49   98  196  392  784 1568 3136
'        G#    26   52  104  208  415  831 1661 3322
'        A     27   55  110  220  440  880 1760 3520
'        A#    29   58  116  233  466  932 1865 3729
'        B     31   62  123  245  494  988 1975 3951
 

#ifdef __FB_WIN32__ 
  #include once "windows.bi"
#endif

Sub Sound_DOS_LIN(Byval freq As Uinteger, dur As Uinteger)
  Dim t As Double
  Dim fixed_freq As uShort
  
    fixed_freq = 1193181 \ freq
    
    ASM
      mov  dx, &H61                  ' turn speaker on
      in   al, dx
      or   al, &H03
      out  dx, al
      mov  dx, &H43                  ' get the timer ready
      mov  al, &HB6
      out  dx, al
      mov  ax, word ptr [fixed_freq] ' move freq to ax
      mov  dx, &H42                  ' port to out
      out  dx, al                    ' out low order
      xchg ah, al                   
      out  dx, al                    ' out high order
    End ASM
    
    t = Timer
    While ((Timer - t) * 1000) < dur ' wait for out specified duration
      Sleep(1)
    Wend
    
    ASM
      mov  dx, &H61                  ' turn speaker off
      in   al, dx
      and  al, &HFC
      out  dx, al
    End ASM
    
End Sub

Sub Sound(Byval freq As Uinteger, dur As Uinteger)
  #ifndef __FB_WIN32__
    ' If not windows Then call the asm version.
    Sound_DOS_LIN(freq, dur)
  #else
    ' If Windows
    Dim osv As OSVERSIONINFO
    
      osv.dwOSVersionInfoSize = sizeof(OSVERSIONINFO)
      GetVersionEx(@osv)
      
      Select Case osv.dwPlatformId
        Case VER_PLATFORM_WIN32_NT       
          ' If NT then use Beep from API
          Beep_(freq, dur)
        Case Else
          ' If not on NT then use the same as DOS/Linux
          Sound_DOS_LIN(freq, dur)
      End Select
  #endif
End Sub


SUB Play (Melodie  AS STRING)
' Spielen einer Melodie. Subset aus dem PLAY-Befehls von QBASIC.
' Melodie ist ein String aus folgenden Kommandos (Gross- oder Kleinbuchstaben):
' - Tn = Tempo = Anzahl von Viertelnoten pro Minute (n = 32 bis 255, 
'   Default 120). Beispiel: "T180"
' - On = aktuelle Oktave (n = 0 bis 5). Beispiel: "O3"
' - < = eine Oktave tiefer
' - > eine Oktave hoeher
' - C, D, E, F, G, A, B: Noten in der aktuellen Oktave. Moegliche Suffixe:
'   + oder # Erhoehung um einen halben Ton. Beispiel: "C#" 
'   - Erniedrigung um einen halben Ton. Beispiel: "D-"
' - . = Eineinhalbfache Notenlaenge. Beispiele: "C.", "C+." oder "D-."
' - Ln = Laenge einer Note (n = 1 bis 64,  1 = ganze Note, 2 = halbe Note, 
'   4 = Viertelnote usw). Beispiel: "L4". (Individuelle Notenlaengen (Beispiel "C4")
'   werden nicht unterstuetzt.)
' - Pn = Laenge einer Pause (n = 1 bis 64,  1 = ganze Pause, 2 = halbe Pause, 
'   4 = Viertelpause usw). Beispiel: "P4"
' Beispiel: Alle meine Entchen schwimmen auf dem See 
' Play("O3 L8 CDEF L4 GG O3 L8 AAAA L2 G")

  'Tonfrequenzen in Hertz:
  DIM AS INTEGER C(5)   = { 65, 131, 262, 523, 1046, 2093}, _
                 Cis(5) = { 69, 139, 277, 554, 1109, 2217}, _
                 D(5)   = { 73, 147, 294, 587, 1175, 2349}, _
                 Dis(5) = { 78, 155, 311, 622, 1244, 2489}, _
                 E(5)   = { 82, 165, 330, 659, 1328, 2637}, _
                 F(5)   = { 87, 175, 349, 698, 1397, 2794}, _
                 Fis(5) = { 92, 185, 370, 740, 1480, 2960}, _
                 G(5)   = { 98, 196, 392, 784, 1568, 3136}, _
                 Gis(5) = {104, 208, 415, 831, 1661, 3322}, _
                 A(5)   = {110, 220, 440, 880, 1760, 3520}, _
                 Ais(5) = {116, 233, 466, 932, 1865, 3729}, _
                 B(5)   = {123, 245, 494, 988, 1975, 3951}
                 
  DIM AS STRING s, s1, s2
  DIM AS INTEGER k, Wert, Okt, LWert, Tempo, Pause, freq, dur1, dur2, dur
  
  'Default-Werte:
  Okt = 3
  LWert = 4
  Tempo = 120
  dur = 60000 / (LWert * Tempo)
  dur1 = (7 / 8) * dur
  dur2 = dur - dur1
  
  s = TRIM(Melodie)
  
  'Alle Leerzeichen entfernen (repariert Fehleingaben wie "L 16" oder "O 3"):
  DO
    k = INSTR(s, " ")
    s = LEFT(s, k - 1) + RIGHT(s, LEN(s) - k)
  LOOP UNTIL k = 0
  
  DO
    
    k = INSTR(s, ANY "OCDEFGABLPTocdefgablpt<>")
    'Eventuellen Schrott vorn entfernen (kann durch Formatfehler entstehen,
    'z.B. "C32", oder ungueltige Zeichen)
    IF k > 1 THEN s = RIGHT(s, LEN(s) - k + 1)
    s1 = UCASE(LEFT(s, 1))
    s2 = ""
                
    SELECT CASE s1
            
    CASE "T", "L", "P"
      k = 0
      DO
        k = k + 1
        s2 = MID(s, 1 + k, 1)
        'Wenn s2 eine Ziffer 0 bis 9 ist, an "T" dranhaengen:
        IF ASC(s2) >=48 AND ASC(s2) <=57 THEN s1 = s1 + s2
      LOOP UNTIL ASC(s2) < 48 OR ASC(s2) > 57
      IF LEN(s1) > 1 THEN
        Wert = VAL(RIGHT(s1, LEN(s1) - 1))
        IF LEFT(s1, 1) = "T" THEN
          Tempo = Wert
          IF Tempo < 32 THEN Tempo = 32
          IF Tempo > 255 THEN Tempo = 255
          dur = 4 * 60000 / (LWert * Tempo)
          dur1 = (7 / 8) * dur
          dur2 = dur - dur1          
        ELSEIF LEFT(s1, 1) = "L" THEN
          LWert = Wert
          IF LWert > 64 THEN LWert = 64
          IF LWert < 1 THEN LWert = 1
          dur = 4 * 60000 / (LWert * Tempo)
          dur1 = (7 / 8) * dur
          dur2 = dur - dur1
        ELSEIF LEFT(s1, 1) = "P" THEN
          Pause = Wert
          IF Pause > 64 THEN Pause = 64
          IF Pause < 1 THEN Pause = 1        
          'Pause spielen:
          SLEEP 4 * 60000 / (Pause * Tempo)
        END IF
      END IF
      
    CASE "O"
      s2 = MID(s, 2, 1)
      IF ASC(s2) >=48 AND ASC(s2) <=57 THEN
        s1 = s1 + s2
        Okt = VAL(s2)
        IF Okt < 0 THEN Okt = 0
        IF Okt > 5 THEN Okt = 5
      END IF
      
    CASE ">"
      IF Okt < 5 THEN Okt = Okt + 1
      
    CASE "<"
      IF Okt > 0 THEN Okt = Okt - 1
      
    CASE "C", "D", "E", "F", "G", "A", "B"
      s2 = MID(s, 2, 1)
      IF s2 = "+" OR s2 = "#" THEN
        SELECT CASE s1
        CASE "C": freq = Cis(Okt)
        CASE "D": freq = Dis(Okt)
        CASE "E": freq = F(Okt)
        CASE "F": freq = Fis(Okt)
        CASE "G": freq = Gis(Okt)
        CASE "A": freq = Ais(Okt)
        CASE "B": IF Okt < 5 THEN freq = C(Okt + 1)
        END SELECT
        s1 = s1 + s2
      ELSEIF s2 = "-" THEN
        SELECT CASE s1
        CASE "C": IF Okt > 0 THEN freq = B(Okt - 1) 
        CASE "D": freq = Cis(Okt)
        CASE "E": freq = Dis(Okt)
        CASE "G": freq = Fis(Okt)
        CASE "A": freq = Gis(Okt)
        CASE "B": freq = Ais(Okt)
        END SELECT
        s1 = s1 + s2
      ELSE
        SELECT CASE s1
        CASE "C": freq = C(Okt)
        CASE "D": freq = D(Okt)
        CASE "E": freq = E(Okt)
        CASE "F": freq = F(Okt)
        CASE "G": freq = G(Okt)
        CASE "A": freq = A(Okt)
        CASE "B": freq = B(Okt)
        END SELECT
      END IF
      IF s2 = "." OR MID(s, 3, 1) = "." THEN
        'Ton spielen:
        Sound(freq, 1.5 * dur1)
        SLEEP 1.5 * dur2 
        s1 = s1 + "."
      ELSE
        'Ton spielen:
        Sound(freq, dur1)
        SLEEP dur2
      END IF
           
    END SELECT
    
    'Ausgewerteten Teil des Strings s vorn abtrennen:
    s = TRIM(s)
    s = RIGHT(s, LEN(s) - LEN(s1))
    
  LOOP UNTIL LEN(s) = 0
  
END SUB

